// Game:           		Prince of Persia SNES
// Emulator:            snes9x 1.60 [x64]
// Original by:         ThePedMeister, Karlgamer, WinterThunder, Smathlax, GMP & eien86 (DOS version)
// Adapted to SNES by:  Shauing, GMP
// In-Game Timer by:    David from forum.princed.org
// Updated:             2024-02-24
// IGT timing:          YES
//
// Base memory path is everything before final offset
// e.g. in ["snes9x-x64.exe", 0x08D8C38, 0x0579] base path is the ["snes9x-x64.exe", 0x08D8C38].
//
// Base memory paths for versions:
// snes9x-x64 1.60: "snes9x-x64.exe", 0x08D8C38

state("snes9x-x64")
{
	byte Level          : "snes9x-x64.exe", 0x08D8C38, 0x0544;          // next level - shows level 1-20 changes as door is entered
	byte Menu           : "snes9x-x64.exe", 0x08D8C38, 0xE0C8;          // 180 in menu and 0 when game start is clicked
	byte Jaffar         : "snes9x-x64.exe", 0x08D8C38, 0x050B;          // Jaffar HP
	byte Scene          : "snes9x-x64.exe", 0x08D8C38, 0x0579;          // current level - shows level 1-20 changes as scenes end
	byte Hourglass      : "snes9x-x64.exe", 0x08D8C38, 0x0629;          // hourglass present - 2 if present, 0 otherwise [If you interrupt the intro before the hourglass appears then it stays 0 until the next cutscene]
	byte Cutscene       : "snes9x-x64.exe", 0x08D8C38, 0x0A84;          // current cutscene - 06 = bad ending, 07 = good ending. It stays so even after the cutscene ends
	byte Start          : "snes9x-x64.exe", 0x08D8C38, 0x0E5D;          // decoded level - level number decoded from an entered password; it stays 00 if you start the training so it's not exactly the same as the DOS variable
	ushort FrameCount   : "snes9x-x64.exe", 0x08D8C38, 0x052D;          // number of elapsed ticks - 1 minute = 0x1A9 (425) ticks, 1 second = 7 ticks
	byte CharScrn       : "snes9x-x64.exe", 0x08D8C38, 0x0462;          // character screen
	byte MsgTimer       : "snes9x-x64.exe", 0x08D8C38, 0x0518;          // message timer
	byte Checkpoint     : "snes9x-x64.exe", 0x08D8C38, 0x0627;          // checkpoint on/off

	int SNbelow         : "snes9x-x64.exe", 0x08D8C38, 0x0605;          // variables that cover one of the arrays used for collision detection.
	int SNbelowplus4    : "snes9x-x64.exe", 0x08D8C38, 0x0609;          // SNbelow+4
	short SNbelowplus8  : "snes9x-x64.exe", 0x08D8C38, 0x060D;          // SNbelow+8

	ushort FrameCount             : "snes9x-x64.exe", 0x008D8C38, 0x052D; // 7E:052D; elapsed frames (8 video frames each); stops on level 20
	byte FrameCountHalf           : "snes9x-x64.exe", 0x008D8C38, 0x052F; // 7E:052F; elapsed half frames (0 or 1) (4 video frames each)
	ushort FrameCountThisLevel    : "snes9x-x64.exe", 0x008D8C38, 0x054B; // 7E:054B; elapsed frames on this level
	byte FrameCountHalfThisLevel  : "snes9x-x64.exe", 0x008D8C38, 0x054D; // 7E:054D; elapsed half frames (0 or 1) + overflow bits multiplied by 2
	ushort FrameCountLevelStart   : "snes9x-x64.exe", 0x008D8C38, 0x054E; // 7E:054E; elapsed frames when the level started; does not work on level 1!
	byte FrameCountHalfLevelStart : "snes9x-x64.exe", 0x008D8C38, 0x0550; // 7E:0550; elapsed half frames (0 or 1) + overflow bits multiplied by 2
}

startup
{
	vars.calculate_time = (Func<int,int,TimeSpan>)delegate(int FrameCount, int FrameCountHalf)
	{
		// 1 minute = 0x01A9 frames, 1 second = 7 frames.
		// 1 frame = 8 video frames, 1 video frame = 1/50 seconds (PAL) or 1/60 seconds (NTSC)
		// So 1 minute is slightly more than 60 seconds, and they don't even match real time.

		int overflow_bits = (FrameCountHalf>>1)<<16;
		int half_frame = FrameCountHalf & 1;

		return TimeSpan.FromMinutes( ((double)(overflow_bits + FrameCount) + 0.5 * (double)half_frame) / (double)0x01A9);
	};
}

reset
{
	return (old.Menu != 180 && current.Menu == 180 && current.Level == 0);
}

start
{
	// start when menu goes from 180 to 0
	vars.JaffarDied = false;
	return (old.Menu == 180 && current.Menu == 0);
}

split
{
	if(current.Level == 20 && old.Jaffar != 0 && current.Jaffar == 0){
		vars.JaffarDied = true;
	}
	bool skipSplit = ((old.Level == 19 && !vars.JaffarDied) || (old.Level == 20 && current.Level == 19));
	return((old.Level == current.Level - 1) && !skipSplit);
}

gameTime
{
	if (current.Scene+1 >= 20)
	{
		// The main timer stops on level 20, but the per-level counter keeps counting.
		// PROBLEM: FrameCountThisLevel resets when you are warped back to level 20.
		return vars.calculate_time(current.FrameCountLevelStart, current.FrameCountHalfLevelStart)
		     + vars.calculate_time(current.FrameCountThisLevel , current.FrameCountHalfThisLevel );
	}
	else
	{
		return vars.calculate_time(current.FrameCount, current.FrameCountHalf);
	}
}

isLoading
{
	// Force Game Time to show the actual Game Time...
	return true;
}
